@@ -34,7 +34,7 @@ class CausalBayesianNetwork(BayesianModel):
             self.cbn = cbn
             self.domain: Dict[str, List[Outcome]] = {}
 
-        def __setitem__(self, variable: str, relationship: Relationship, sync_state_names: bool = True) -> None:
+        def __setitem__(self, variable: str, relationship: Relationship) -> None:
 
             # Update the keys
             if variable in self.keys():
@@ -56,9 +56,9 @@ class CausalBayesianNetwork(BayesianModel):
             if not (old_domain and old_domain == self.domain[variable]):
                 for child in self.cbn.get_children(variable):
                     if child in self.keys():
-                        self.__setitem__(child, self[child], sync_state_names=False)  # type: ignore
-                if sync_state_names:
-                    self.sync_state_names()
+                        self.__setitem__(child, self[child])  # type: ignore
+
+            self.sync_state_names()
 
         def __delitem__(self, variable: str) -> None:
             super().__delitem__(variable)
@@ -90,15 +90,6 @@ class CausalBayesianNetwork(BayesianModel):
         self.model = self.Model(self)
         super().__init__(ebunch=edges)
 
-        self._lowercase_to_variable: Dict[str, str] = {}
-        for node in self.nodes:
-            if node.lower() in self._lowercase_to_variable:
-                raise ValueError(
-                    f'Name conflict: Both "{node}" and "{self._lowercase_to_variable[node.lower()]}" '
-                    f'have the same lowercase "{node.lower()}".'
-                )
-            self._lowercase_to_variable[node.lower()] = node
-
     def remove_edge(self, u: str, v: str) -> None:
         """removes an edge u to v that exists from the CBN"""
         super().remove_edge(u, v)
@@ -116,22 +107,26 @@ class CausalBayesianNetwork(BayesianModel):
         Add the given CPDs and initialize StochasticFunctionCPDs
         """
         for cpd in cpds:
-            self.model.__setitem__(cpd.variable, cpd, sync_state_names=False)  # type: ignore
-        self.model.update(relationships)  # TODO: Prevent sync_state_names
-        self.model.sync_state_names()
+            self.model[cpd.variable] = cpd  # type: ignore
+        self.model.update(relationships)
 
     def remove_cpds(self, *cpds: Union[str, TabularCPD]) -> None:
         for cpd in cpds:
             del self.model[cpd.variable if isinstance(cpd, TabularCPD) else cpd]
 
-    def _fix_lowercase_variables(self, outcome_dict: Dict[str, Outcome]) -> None:
+    def is_structural_causal_model(self) -> bool:
+        """Check whether self represents a Structural Causal Model (SCM).
+
+        Nodes without parents are interpreted as exogenous, and are allowed to have any
+        distribution. All other nodes are interpreted as endogenous, and are therefore
+        required to have degenerate CPD tables containing only 0 and 1s.
         """
-        Outcomes are sometimes specified in terms of lowercase versions of variable names.
-        They need to be converted, before passed to factor.query
-        """
-        for var in set(outcome_dict).intersection(self._lowercase_to_variable):
-            outcome_dict[self._lowercase_to_variable[var]] = outcome_dict[var]
-            del outcome_dict[var]
+        for node in self:
+            if self.get_parents(node):
+                for probability in self.get_cpds(node).values.flatten():
+                    if not (np.isclose(probability, 0) or np.isclose(probability, 1)):
+                        return False
+        return True
 
     def query(
         self, query: Iterable[str], context: Dict[str, Outcome], intervention: Dict[str, Outcome] = None
@@ -149,8 +144,6 @@ class CausalBayesianNetwork(BayesianModel):
 
         intervention: Interventions to apply. A dictionary mapping node => outcome.
         """
-        self._fix_lowercase_variables(context)
-
         for variable, outcome in context.items():
             if outcome not in self.model.domain[variable]:
                 raise ValueError(f"The outcome {outcome} is not in the domain of {variable}")
@@ -189,7 +182,6 @@ class CausalBayesianNetwork(BayesianModel):
         ----------
         intervention: Interventions to apply. A dictionary mapping node => value.
         """
-        self._fix_lowercase_variables(intervention)
         for variable in intervention:
             for p in self.get_parents(variable):  # remove ingoing edges
                 self.remove_edge(p, variable)
@@ -242,8 +234,8 @@ class CausalBayesianNetwork(BayesianModel):
     def copy(self) -> CausalBayesianNetwork:
         """copy the MACIDBase object"""
         model_copy = self.copy_without_cpds()
-        if self.cpds:
-            model_copy.add_cpds(*[cpd.copy() for cpd in self.cpds])
+        for v in self.model:
+            model_copy.model[v] = self.model[v].copy() if hasattr(self.model[v], "copy") else self.model[v]
         return model_copy
 
     def _get_color(self, node: str) -> Union[np.ndarray, str]:
