@@ -2,7 +2,6 @@ from __future__ import annotations
 
 import inspect
 import itertools
-import types
 from inspect import getsourcelines
 from typing import TYPE_CHECKING, Any, Callable, Dict, Iterator, List, Mapping, Optional, Sequence, Union
 
@@ -19,17 +18,6 @@ class ParentsNotReadyException(ValueError):
     pass
 
 
-def function_copy(f: Callable) -> Callable:
-    """
-    return a function with same code, globals, defaults, closure, and
-    name (or provide a new name)
-    """
-    fn = types.FunctionType(f.__code__, f.__globals__, f.__name__, f.__defaults__, f.__closure__)  # type: ignore
-    # in case f was given attrs (note this dict is a shallow copy):
-    fn.__dict__.update(f.__dict__)
-    return fn
-
-
 class StochasticFunctionCPD(TabularCPD):
     """
     StochasticFunctionCPD class used to specify relationship between variables with a stochastic
@@ -72,8 +60,8 @@ class StochasticFunctionCPD(TabularCPD):
         stochastic_function: A stochastic function that maps parent outcomes to a distribution
         over outcomes for this variable (see doc-string for class).
         The different parents are identified by name: the arguments to the function must
-        be lowercase versions of the names of the parent variables. For example, if X has
-        parents Y, S1, and Obs, the arguments to function must be y, s1, and obs.
+        match the names of the parent variables. For example, if X has
+        parents Y, S1, and Obs, the arguments to function must be Y, S1, and Obs.
 
         domain: An optional specification of the variable's domain.
             Must include all values this variable can take as a result of its function.
@@ -161,11 +149,10 @@ class StochasticFunctionCPD(TabularCPD):
         sig = inspect.signature(self.stochastic_function).parameters
         arg_kinds = [arg_kind.kind.name for arg_kind in sig.values()]
         args = set(sig)
-        lower_case_parents = {p.lower() for p in self.cbn.get_parents(self.variable)}
-        if "VAR_KEYWORD" not in arg_kinds and args != lower_case_parents:
+        if "VAR_KEYWORD" not in arg_kinds and args != set(self.cbn.get_parents(self.variable)):
             raise ValueError(
                 f"function for {self.variable} mismatch parents on"
-                f" {args.symmetric_difference(lower_case_parents)}, "
+                f" {args.symmetric_difference(set(self.cbn.get_parents(self.variable)))}, "
             )
 
     def parent_values(self) -> Iterator[Dict[str, Outcome]]:
@@ -177,7 +164,7 @@ class StochasticFunctionCPD(TabularCPD):
         except KeyError:
             raise ParentsNotReadyException(f"Parent {p} of {self.variable} not yet instantiated")
         for parent_values in itertools.product(*parent_values_list):
-            yield {p.lower(): parent_values[i] for i, p in enumerate(self.cbn.get_parents(self.variable))}
+            yield {p: parent_values[i] for i, p in enumerate(self.cbn.get_parents(self.variable))}
 
     def possible_values(self) -> List[Outcome]:
         """The possible values this variable can take, given the values the parents can take"""
@@ -258,7 +245,9 @@ def discrete_uniform(domain: List[Outcome]) -> Dict[Outcome, float]:
     return {outcome: 1 / len(domain) for outcome in domain}
 
 
-def noisy_copy(value: Outcome, probability: float = 0.9, domain: List[Outcome] = None) -> Dict[Outcome, float]:
+def noisy_copy(
+    value: Outcome, probability: float = 0.9, domain: List[Outcome] = None
+) -> Dict[Outcome, Optional[float]]:
     """specify a variable's CPD as copying the value of some other variable with a certain probability."""
     dist = dict.fromkeys(domain) if domain else {}
     dist[value] = probability
