@@ -52,12 +52,10 @@ class MACIDBase(CausalBayesianNetwork):
     """
 
     class Model(CausalBayesianNetwork.Model):
-        def __setitem__(
-            self, variable: str, relationship: Union[Relationship, Sequence], sync_state_names: bool = False
-        ) -> None:
+        def __setitem__(self, variable: str, relationship: Union[Relationship, Sequence]) -> None:
             if isinstance(relationship, (DecisionDomain, Sequence)) and variable not in self.cbn.decisions:
                 raise ValueError(f"trying to add DecisionDomain to non-decision node {variable}")
-            super().__setitem__(variable, relationship, sync_state_names)
+            super().__setitem__(variable, relationship)
 
         def to_tabular_cpd(self, variable: str, relationship: Union[Relationship, Sequence[Outcome]]) -> TabularCPD:
             if isinstance(relationship, Sequence):
@@ -153,8 +151,6 @@ class MACIDBase(CausalBayesianNetwork):
 
         intervention: Interventions to apply. A dictionary mapping node => outcome.
         """
-
-        self._fix_lowercase_variables(context)
         for variable, outcome in context.items():
             if outcome not in self.get_cpds(variable).domain:
                 raise ValueError(f"The outcome {outcome} is not in the domain of {variable}")
@@ -292,7 +288,7 @@ class MACIDBase(CausalBayesianNetwork):
             idx = 0
             for i, parent in enumerate(parents):
                 name_to_no: Dict[Outcome, int] = self.get_cpds(parent).name_to_no[parent]
-                idx += name_to_no[pv[parent.lower()]] * int(np.product(parent_cardinalities[:i]))
+                idx += name_to_no[pv[parent]] * int(np.product(parent_cardinalities[:i]))
             assert 0 <= idx <= number_of_decision_contexts
             return idx
 
@@ -374,13 +370,7 @@ class MACIDBase(CausalBayesianNetwork):
     def remove_all_decision_rules(self) -> None:
         """Remove the decision rules from all decisions in the (MA)CID"""
         for d in self.decisions:
-            cpd = self.get_cpds(d)
-            if cpd is None:
-                raise ValueError(f"decision {d} has not yet been assigned a domain.")
-            elif isinstance(cpd, DecisionDomain):
-                continue
-            else:
-                self.add_cpds(DecisionDomain(d, self, cpd.domain))
+            self.model[d] = self.model.domain[d]
 
     def impute_optimal_decision(self, decision: str) -> None:
         """Impute an optimal policy to the given decision node"""
@@ -408,12 +398,12 @@ class MACIDBase(CausalBayesianNetwork):
 
         @lru_cache(maxsize=1000)
         def cond_exp_policy(**pv: Outcome) -> float:
-            if y.lower() in pv:
-                return pv[y.lower()]  # type: ignore
+            if y in pv:
+                return pv[y]  # type: ignore
             else:
                 return copy.expected_value([y], pv)[0]
 
-        self.add_cpds(StochasticFunctionCPD(decision, cond_exp_policy, self, label="cond_exp({})".format(y)))
+        self.add_cpds(**{decision: cond_exp_policy})
 
     # Wrapper around DAG.active_trail_nodes to accept arbitrary iterables for observed.
     # Really, DAG.active_trail_nodes should accept Sets, especially since it does
